<?php

class PayshopCard extends WC_Payment_Gateway 
{
    /**
     * @var PayshopCreateCharge
     */
    private $payshopCreateCharge;

    /**
     * @var PayshopCreateInstrument
     */
    private $payshopCreateInstrument;

    /**
     * Define the payment gateway properties
     */
    public function __construct()
    {
        $this->id = 'payshop_card';
        $this->icon = PAYSHOP_PLUGIN_URL . 'templates/img/visa_mc.png'; 
        $this->has_fields = true; 
        $this->method_title = __('Payshop Card', 'payshop');
        $this->method_description = __('Debit and credit card on Payshop Online Payments', 'payshop'); 

        $this->supports = [
            'products'
        ];

        $this->init_form_fields();
        $this->init_settings();
        $this->title = $this->get_option('payshop_card_title');
        $this->description = $this->get_option('payshop_card_description');
        $this->enabled = $this->get_option('payshop_card_enabled');
     
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);
        add_action('wp_enqueue_scripts', [$this, 'payment_scripts']);

        $this->payshopCreateCharge = new PayshopCreateCharge();
        $this->payshopCreateInstrument = new PayshopCreateInstrument();
    }

    /**
     * Define configuration fields for the payment gateway
     * 
     * @return void
     */
    public function init_form_fields()
    {
        $this->form_fields = array(
            'payshop_card_enabled' => array(
                'title'       => __('Enable/Disable', 'payshop'),
                'label'       => __('Enable Card Payment', 'payshop'),
                'type'        => 'checkbox',
                'description' => '',
                'default'     => 'no'
            ),
            'payshop_payment_in_two_steps' => array(
                'title'       => __('Payment in two steps', 'payshop'),
                'label'       => __('Enable Payment in two steps, first capture the amount and then merchant confirms the value', 'payshop'),
                'type'        => 'checkbox',
                'description' => '',
                'default'     => 'no'
            ),
            'payshop_card_title' => array(
                'title'       => __('Title'),
                'type'        => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'payshop'),
                'default'     => __('Credit or debit card', 'payshop'),
            ),
            'payshop_card_description' => array(
                'title'       => __('Description'),
                'type'        => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'payshop'),
                'default'     => __('Pay with your card via with Payshop Online Payments.', 'payshop'),
            ),
        );
    }

    /**
     * Enqueue the scripts and styles
     * 
     * @return void
     */
    public function payment_scripts() 
    {
        if (!is_cart() && !is_checkout() && !isset($_GET['pay_for_order'])) {
            return;
        }

        if ('no' === $this->enabled) {
            return;
        }

        wp_register_script('payshop_card', PAYSHOP_PLUGIN_URL . 'templates/js/card.js', ['jquery']);
        wp_enqueue_script('payshop_card');
    }
    
    /**
     * Display the payment fields on the checkout page
     * 
     * @return void
     */
    public function payment_fields()
    {
        if ($this->description) {
            if (get_option('payshop_environment') == 'sandbox') {
                $this->description .= '<br>' . __('TEST MODE ENABLED.', 'payshop');
            }

            echo wpautop(wp_kses_post($this->description));
        }

        do_action('woocommerce_credit_card_form_start', $this->id);
        require_once PAYSHOP_PLUGIN_DIR . '/templates/views/front/card.php';
        do_action('woocommerce_credit_card_form_end', $this->id);
    }

    /**
     * Validate the payment fields
     * 
     * @return bool
     */
    public function validate_fields()
    {
        $cardData = $this->getCardData();

        $isValid = true;

        if (!$cardData['number'] || strlen($cardData['number']) != 16) {
            wc_add_notice(__('Card number is invalid', 'payshop'));
            $isValid = false;
        }

        if (!$cardData['name']) {
            wc_add_notice(__('Card holder name is required', 'payshop'));
            $isValid = false;
        }

        $invalidExpiration = !$cardData['expiration_month'] ||
                             !$cardData['expiration_year'] ||
                             strlen($cardData['expiration_month']) < 2 ||
                             strlen($cardData['expiration_year']) < 4;

        if ($invalidExpiration) {
            wc_add_notice(__('Card expiration is invalid', 'payshop'));
            $isValid = false;
        }

        if (!$cardData['cvc'] || strlen($cardData['cvc']) != 3) {
            wc_add_notice(__('Card security code is invalid', 'payshop'));
            $isValid = false;
        }

        return $isValid;
    }

    /**
     * Process the payment
     * 
     * @param int $orderId
     * @return array|void
     */
    public function process_payment($orderId)
    {
        try {
            $order = wc_get_order($orderId);

            $order->update_meta_data('_payshop_payment_in_two_steps', $this->get_option('payshop_payment_in_two_steps'), true);
            $order->save();

            $chargeId = $this->payshopCreateCharge->execute(
                'card', 
                $orderId, 
                $this->get_return_url($order)
            );

            $instrument = $this->createInstrument($chargeId, $order);
            $this->checkInstrument($instrument);

            return [
                'result' => 'success',
                'redirect' => $instrument['redirect']['url']
            ];
        } catch (\Throwable $th) {
            wc_add_notice($th->getMessage(), 'error');
            return;
        }
    }
    
    /**
     * Create instrument
     * 
     * @param string $chargeId
     * @param WC_Order $order
     * @return array
     */
    private function createInstrument($chargeId, $order)
    {
        $instrumentData = ["charge" => $chargeId] + $this->getCardData();

        $paymentInTwoSteps = $order->get_meta('_payshop_payment_in_two_steps', true);

        if ($paymentInTwoSteps == 'yes') {
            $instrumentData['capture_on_creation'] = false;
            $instrumentData['instrument_params'] = ['capture_on_creation' => false];
        }

        return $this->payshopCreateInstrument->execute($instrumentData);
    }

    /**
     * Get the card data
     * 
     * @return array
     */
    private function getCardData()
    {
        return [
            'number' => isset($_POST['card-number']) ? str_replace(' ', '', $_POST['card-number']) : '',
            'name' => isset($_POST['card-holder-name']) ? $_POST['card-holder-name'] : '',
            'expiration_month' => isset($_POST['card-expiration']) ? substr($_POST['card-expiration'], 0, 2) : '',
            'expiration_year' => isset($_POST['card-expiration']) ? substr($_POST['card-expiration'], 3, 4) : '',
            'cvc' => isset($_POST['card-security-code']) ? $_POST['card-security-code'] : ''
        ];
    }

    /**
     * Check instrument
     * 
     * @param array $instrument
     * @return void
     * @throws \Exception
     */
    private function checkInstrument($instrument)
    {
        $isNotPending = $instrument['status'] !== 'pending';
        $notHasRedirectUrl = !isset($instrument['redirect']['url']);

        if ($isNotPending || $notHasRedirectUrl) {
            throw new Exception(__('Error processing payment', 'payshop'));
        }
    }
}

