<?php

class PayshopMBWay extends WC_Payment_Gateway 
{
    /**
     * @var PayshopCreateCharge
     */
    private $payshopCreateCharge;

    /**
     * @var PayshopCreateInstrument
     */
    private $payshopCreateInstrument;

    /**
     * @var PayshopUpdateOrder
     */
    private $payshopUpdateOrder;

    /**
     * Define the payment gateway properties
     */
    public function __construct()
    {
        $this->id = 'payshop_mbway';
        $this->icon = PAYSHOP_PLUGIN_URL . 'templates/img/mbway.png'; 
        $this->has_fields = true; 
        $this->method_title = __('MBWay', 'payshop');
        $this->method_description = __('Pay with MBWay', 'payshop'); 

        $this->supports = [
            'products'
        ];

        $this->init_form_fields();
        $this->init_settings();
        $this->title = $this->get_option('payshop_mbway_title');
        $this->description = $this->get_option('payshop_mbway_description');
        $this->enabled = $this->get_option('payshop_mbway_enabled');
     
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);
        add_action('wp_enqueue_scripts', [$this, 'payment_scripts']);

        $this->payshopCreateCharge = new PayshopCreateCharge();
        $this->payshopCreateInstrument = new PayshopCreateInstrument();
        $this->payshopUpdateOrder = new PayshopUpdateOrder();
    }

    /**
     * Define configuration fields for the payment gateway
     * 
     * @return void
     */
    public function init_form_fields()
    {
        $this->form_fields = array(
            'payshop_mbway_enabled' => array(
                'title'       => __('Enable/Disable', 'payshop'),
                'label'       => __('Enable MBWay Payment', 'payshop'),
                'type'        => 'checkbox',
                'description' => '',
                'default'     => 'no'
            ),
            'payshop_mbway_title' => array(
                'title'       => __('Title'),
                'type'        => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'payshop'),
                'default'     => __('MBWay', 'payshop'),
            ),
            'payshop_mbway_description' => array(
                'title'       => __('Description'),
                'type'        => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'payshop'),
                'default'     => __('Pay with MBWay', 'payshop'),
            ),
        );
    }

    /**
     * Enqueue the scripts and styles
     * 
     * @return void
     */
    public function payment_scripts() 
    {
        if (!is_cart() && !is_checkout() && !isset($_GET['pay_for_order'])) {
            return;
        }

        if ('no' === $this->enabled) {
            return;
        }

        wp_register_script('payshop_mbway', PAYSHOP_PLUGIN_URL . 'templates/js/mbway.js', ['jquery']);
        wp_enqueue_script('payshop_mbway');
    }
    
    /**
     * Display the payment fields on the checkout page
     * 
     * @return void
     */
    public function payment_fields()
    {
        if ($this->description) {
            if (get_option('payshop_environment') == 'sandbox') {
                $this->description .= '<br>' . __('TEST MODE ENABLED.', 'payshop');
            }

            echo wpautop(wp_kses_post($this->description));
        }

        do_action('woocommerce_mbway_form_start', $this->id);
        require_once PAYSHOP_PLUGIN_DIR . '/templates/views/front/mbway.php';
        do_action('woocommerce_mbway_form_end', $this->id);
    }

    /**
     * Validate the payment fields
     * 
     * @return bool
     */
    public function validate_fields()
    {
        $cardData = $this->getMBWayData();

        if (!$cardData['phone'] || strlen($cardData['phone']) != 9) {
            wc_add_notice(__('Phone number is invalid', 'payshop'));
            return false;
        }

        return true;
    }

    /**
     * Process the payment
     * 
     * @param int $orderId
     * @return array|void
     */
    public function process_payment($orderId)
    {
        try {
            $order = wc_get_order($orderId);

            $chargeId = $this->payshopCreateCharge->execute(
                'mbway', 
                $orderId, 
                $this->get_return_url($order)
            );

            $instrumentId = $this->createInstrument($chargeId)['id'];

            $this->payshopUpdateOrder->execute(
                $orderId,
                "on-hold",
                $chargeId, 
                $instrumentId
            );

            return [
                'result' => 'success',
                'redirect' => $this->get_return_url($order)
            ];
        } catch (\Throwable $th) {
            wc_add_notice($th->getMessage(), 'error');
            return;
        }
    }
    
    /**
     * Create instrument
     * 
     * @param string $chargeId
     * @param WC_Order $order
     * @return array
     */
    private function createInstrument($chargeId)
    {
        $instrumentData = ["charge" => $chargeId] + $this->getMBWayData();

        return $this->payshopCreateInstrument->execute($instrumentData);
    }

    /**
     * Get the card data
     * 
     * @return array
     */
    private function getMBWayData()
    {
        return [
            'phone' => isset($_POST['phone']) ? str_replace(' ', '', $_POST['phone']) : '',
        ];
    }

}

